package org.jboss.logging;

import java.util.List;

/**
 * MBean interface.
 */
public interface JDKLoggerServiceMBean extends org.jboss.system.ServiceMBean {

	// default object name
	public static final javax.management.ObjectName OBJECT_NAME = org.jboss.mx.util.ObjectNameFactory
			.create("jboss.system:type=JDKLoggerService,service=Logging");

	/**
	 * Get the refresh period.
	 */
	int getRefreshPeriod();

	/**
	 * Set the refresh period.
	 */
	void setRefreshPeriod(int refreshPeriod);

	/**
	 * Get the JDK Logger configuration URL.
	 */
	java.net.URL getConfigurationURL();

	/**
	 * Set the JDK Logger configuration URL.
	 */
	void setConfigurationURL(java.net.URL url);

	/**
	 * Sets the level for a logger of the give name.
	 * <p>
	 * Values are trimmed before used.
	 * 
	 * @param name
	 *            The name of the logger to change level
	 * @param levelName
	 *            The name of the level to change the logger to.
	 */
	void setLoggerLevel(java.lang.String name, java.lang.String levelName);

	/**
	 * Sets the levels of each logger specified by the given comma separated
	 * list of logger names.
	 * 
	 * @see #setLoggerLevel
	 * @param list
	 *            A comma separated list of logger names.
	 * @param levelName
	 *            The name of the level to change the logger to.
	 */
	void setLoggerLevels(java.lang.String list, java.lang.String levelName);

	/**
	 * Gets the level of the logger of the give name.
	 * 
	 * @param name
	 *            The name of the logger to inspect.
	 */
	java.lang.String getLoggerLevel(java.lang.String name);

	/**
	 * Force the logging system to reconfigure.
	 */
	void reconfigure() throws java.io.IOException;

	/**
	 * Hack to reconfigure and change the URL. This is needed until we have a
	 * JMX HTML Adapter that can use PropertyEditor to coerce.
	 * 
	 * @param url
	 *            The new configuration url
	 */
	void reconfigure(java.lang.String url) throws java.io.IOException,
			java.net.MalformedURLException;

	/**
	 * Returns the list of currently registered loggers.
	 * 
	 * @return A list of <tt>String</tt> each of which is a currently
	 *         registered <tt>Logger</tt> name.
	 */
	List<String> getLoggerNames();

}