package org.jboss.logging;

import java.net.MalformedURLException;
import java.util.logging.Logger;

import org.jboss.embedded.Bootstrap;
import org.jboss.logging.JDKLoggerService;
import org.jboss.logging.JDKLoggerServiceMBean;
import org.jboss.mx.util.MBeanServerLocator;
import org.jboss.virtual.plugins.context.vfs.AssembledContextFactory;
import org.jboss.virtual.plugins.context.vfs.AssembledDirectory;

import fr.biomerieux.embeddableejb3tools.EmbeddableEJB3TestCase;

/**
 * @author walmetjc
 */
public class JDKLoggerTestCase extends EmbeddableEJB3TestCase {

	@Override
	public void deployable() {
		try {
			JDKLoggerServiceMBean mbean = new JDKLoggerService();
			addDeployMBean(JDKLoggerService.OBJECT_NAME, mbean);
		} catch (MalformedURLException e) {
			throw new RuntimeException("Unexpected Exception !!!");
		}
	}

	@Override
	protected void setUp() throws Exception {

		// Workaround: When topics and queues are undeploy, they cannot be
		// redeployed.
		Bootstrap bootstrap = Bootstrap.getInstance();
		if (!bootstrap.isStarted()) {
			startup();
			AssembledDirectory jar = AssembledContextFactory.getInstance()
					.create("mbean.jar");

			jar.addResource("logger-service.xml");

			bootstrap.deploy(jar);
		}

		super.setUp();

	}

	/**
	 */
	public void testLoggerService() throws Exception {
		// Check the configuration url
		assertEquals("Bad configuration url !", "resource:logging.properties",
				getAttribute("ConfigurationURL").toString());

		// Add a logger hadler for test and check that hadler is empty
		String loggerName = getClass().getSimpleName();
		Logger log = Logger.getLogger(loggerName);
		ValidationHandler vh = new ValidationHandler();

		// log.setUseParentHandlers(false);
		log.addHandler(vh);
		assertEquals("Bad number of log !", 0, vh.getErrorCount());

		// log two messages, one with an Info level and the other with a severe
		// level. Check that messages have be logged.
		log.info("Log info !");
		log.severe("Log severe !");

		assertEquals("Bad number of log !", 2, vh.getErrorCount());

		// Change the logger configuration.
		// Check the configuration url
		invoke("reconfigure", new String[] { "resource:logging2.properties" },
				new String[] { "java.lang.String" });
		vh = new ValidationHandler();
		log.addHandler(vh);
		assertEquals("Bad configuration url !", "resource:logging2.properties",
				getAttribute("ConfigurationURL").toString());

		// log two messages, one with an Info level and the other with a severe
		// level. Check that only severe message has been logged.
		log.info("Log info !");
		log.severe("Log severe !");

		assertEquals("Bad number of log !", 1, vh.getErrorCount());

	}

	public void testLoggerLevel() throws Exception {
		// Check the default level read from logging.properties
		String loggerName = getClass().getSimpleName();
		Logger log = Logger.getLogger(loggerName);
		String level = invoke("getLoggerLevel", new String[] { loggerName },
				new String[] { "java.lang.String" }).toString();
		assertEquals("Bad level !", "FINEST", level);

		// Change the logger level. Check that level has been updated
		invoke("setLoggerLevel", new String[] { loggerName, "SEVERE" },
				new String[] { "java.lang.String", "java.lang.String" });
		level = invoke("getLoggerLevel", new String[] { loggerName },
				new String[] { "java.lang.String" }).toString();
		assertEquals("Bad level !", "SEVERE", level);

		// Send a message with a lower level than logger level. Check that
		// message is not logged
		ValidationHandler vh = new ValidationHandler();
		log.addHandler(vh);
		assertEquals("Bad number of log !", 0, vh.getErrorCount());
		log.info("info");
		assertEquals("Bad number of log !", 0, vh.getErrorCount());

		// Send a message with a SEVERE level than logger level. Check that
		// message is logged
		log.severe("severe");
		assertEquals("Bad number of log !", 1, vh.getErrorCount());
	}

	private Object invoke(String method, String[] params, String[] classes)
			throws Exception {
		return MBeanServerLocator.locateJBoss().invoke(
				JDKLoggerServiceMBean.OBJECT_NAME, method, params, classes);
	}

	private Object getAttribute(String attribute) throws Exception {
		return MBeanServerLocator.locateJBoss().getAttribute(
				JDKLoggerServiceMBean.OBJECT_NAME, attribute);
	}

	@Override
	protected void tearDown() throws Exception {
		invoke("reconfigure", new String[] { "resource:logging.properties" },
				new String[] { "java.lang.String" });
		super.tearDown();
	}
}
